/*
 * This file is part of the Baloo KDE project.
 * Copyright 2018  Michael Heidelbach <heidelbach@web.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef INDEXERCONFIGTEST_H
#define INDEXERCONFIGTEST_H

#include <QObject>
// #include <QTemporaryDir>
class QTemporaryDir;

namespace Baloo {
namespace Test {
class FileIndexerConfigTest : public QObject
{
    Q_OBJECT
private Q_SLOTS:

    void initTestCase();
    void cleanupTestCase();

    void testShouldFolderBeIndexed();
    void testShouldFolderBeIndexed_data();

    void testShouldFolderBeIndexedHidden();
    void testShouldFolderBeIndexedHidden_data();

    void testShouldBeIndexed();
    void testShouldBeIndexed_data();

    void testShouldBeIndexedHidden();
    void testShouldBeIndexedHidden_data();

    void testShouldExcludeFilterOnFolders();
    void testShouldExcludeFilterOnFolders_data();

private:
    QTemporaryDir* m_mainDir;
    QString m_dirPrefix;
};

}
}
#endif // INDEXERCONFIGTEST_H
